var data,
dataSum, 
header,
h = 400,
w = 150;

/******************************** 
* Data
*********************************/
//auto_mpg_3models is a variable containing the csv file
var parsedCsv = d3.csv.parseRows(auto_mpg_3models);

//// drop headers
header = parsedCsv.shift();
header.shift();
var transposedCsv = d3.zip.apply(this, parsedCsv)
var carNames= transposedCsv.shift();
//// parse float, sum, range
dataSum = [];
dataRange = [];
for (var i = 0; i < transposedCsv.length; i++) {
	dataSum.push(0);
	for (var j = 0; j < transposedCsv[i].length; j++) {
		transposedCsv[i][j] = parseFloat(transposedCsv[i][j]);
		dataSum[i] += transposedCsv[i][j];
	}
	transposedCsv[i].sort(function(a, b){return a - b;});
	dataRange[i] = d3.max(transposedCsv[i]) - d3.min(transposedCsv[i]) + 1;
};
//// reformatted data
data = transposedCsv;

//// return an array of percentages for each bin
function reduceAsPercentages(data, binNum){
    var binSize = (d3.max(data) - d3.min(data) + 1) / binNum;
    var dataSplitted = [];
    var percentArray = [];
    var i;
    for(i=0; i<binNum; i++){
        dataSplitted.push([]);
    }
    for(i=0; i<data.length; i++){
        var binIdx = Math.floor((data[i]-d3.min(data))/binSize);
        dataSplitted[binIdx].push(data[i]);
    }
    for(i=0; i<binNum; i++){
        percentArray.push(Math.round(dataSplitted[i].length/data.length*100));
    }
    return percentArray;
}

//// find the min and the max for each bin
function reduceAsMinMax(data, binNum){
    var binSize = (d3.max(data) - d3.min(data) + 1) / binNum;
    var dataSplitted = [];
    var minMaxArray = [];
    var i;
    for(i=0; i<binNum; i++){
        dataSplitted.push([]);
    }
    for(i=0; i<data.length; i++){
        var binIdx = Math.floor((data[i]-d3.min(data))/binSize);
        dataSplitted[binIdx].push(data[i]);
    }
     for(i=0; i<binNum; i++){
        minMaxArray.push([d3.min(dataSplitted[i]), d3.max(dataSplitted[i])]);
    }
    return minMaxArray;
}

//// filter out duplicates
function unique(array){
    var o = {}, i, l = array.length, r = [];
    for(i=0; i<l;i+=1) o[array[i]] = array[i];
    for(i in o) r.push(o[i]);
    return r;
}

//// convert all to percentages
var dataPercentages = [];
var dataSplittedByBins = [];
var dataBinValues = [];
for(var i=0; i<data.length; i++){
    var binNum = Math.min(7, unique(data[i]).length);
    var percent = reduceAsPercentages(data[i], binNum);
    var minMax = reduceAsMinMax(data[i], binNum);
    dataBinValues.push([]);
    for(var j=0; j<binNum; j++){
        dataBinValues[i].push({"percent": percent[j], "min": minMax[j][0], "max": minMax[j][1]});
    }
    dataBinValues[i].sort(function(a, b){return a.percent - b.percent;});
}
var underlyingData = data;
//// the final data to use
data = dataBinValues;

/******************************** 
* Stripchart table
*********************************/
//// stripchart table
d3.select(".container_div")
	.style("float", "left")
	.append("table")
	.attr("class", "container_table")
	.style("border-collapse", "collapse")
	.style("height", (h+110)+"px")
	.style("width", w+"px")
	.style("background-color","white")

	//// stripchart table columns
	.append("tr")
	.selectAll("td.container_table_col")
	.data(data)
	.enter().append("td")
	.attr("class", "container_table_col")
	.attr("valign", "top")
	.style("height", "100%")
	
	//// strip tables in each stripchart columns     
	.each(function(parentData, parentIdx){
		d3.select(this)  
		.append("table")
		.attr("class", "strip_table")
		.style("width", "100%")
		.style("height", h+"px")
		.style("table-layout", "fixed")
		
		//// strip slices 
		.selectAll("tr.slice_row")
		.data(parentData)
		.enter().append("tr")
		.attr("class", "slice_row")
		.append("td")
		.attr("valign", "top")
		.attr("class", "slice")
		// hardcoded class only on last slice of horsepower
		.classed("other_slice", function(d, i){return (i == parentData.length-1 && header[parentIdx] == "horsepower");})
		// hardcoded class only on last slice of model_year
		.classed("hierarchy", function(d, i){return (header[parentIdx] == "model_year");})
		// hardcoded class only on last slice of origin
		.classed("list", function(d, i){return (header[parentIdx] == "origin");})
		.on("mouseover", function(){d3.select(this).style("background-color", "skyblue");})
		.on("mouseout", function(){
			d3.select(this).style("background-color", function(d, i){
			var grayscale = 175-d.percent*1.5;
			return 'rgb('+grayscale+', '+grayscale+', '+grayscale+')'});})
		.style("overflow", "hidden")
		.style("background-color", function(d, i){
		    var grayscale = 175-d.percent*1.5;
			return 'rgb('+grayscale+', '+grayscale+', '+grayscale+')'})
		.style("height", function(d, i){return d.percent+"%";})

		//// slice labels
		.append("div")
		//.style("height", "15px") //ugly hack
		.style("overflow", "hidden")
		.attr("class", "slice_label")
		.style("height", "10px")
		.style("color", "white")
		//// hardcoded "Others" only on last slice of horsepower
		.text(function(d, i){return d3.select(this.parentNode).classed("other_slice") ?
            "Others" :
            d.percent+"% ("+
            ((d.min == 1) ? "America " : "")+
            ((d.min == 2) ? "Europe " : "")+
            ((d.min == 3) ? "Asia " : "")+
            ((d.min != d.max) ? (d.min+" - "+d.max) : d.min)+
            ")";
		})
		.style("opacity", 0);
	});

	//// strip headers
	d3.selectAll("td.container_table_col")
		.append("div")
		//.style("text-align", "left")
		.attr("class", "strip_header")
		.style("overflow", "hidden")
		.on("mouseover", function(){d3.select(this).style("background-color", "skyblue");})
		.on("mouseout", function(){d3.select(this).style("background-color", "white");})
		.style("border", "1px solid silver")
		.style("padding", "0px")
		.style("background-color", "white")
		.style("height", "110px")
		.append("div")
		.attr("class", "strip_header_label")
		//.style("width", (w/data.length+3)+"px")
		.style("width", "10px")
		.style("margin-top", "80px")
		.style("-webkit-transform", function(d, i){return "rotate(-90deg)";})
		.text(function(d, i){return header[i];});

/******************************** 
* Horizontal expansion
*********************************/		
var focusedStrip = 120,
	unfocusedStrip = w/data.length-1;

//// expand/compress horizontally on header click
d3.selectAll(".strip_header")
	.on("click", onStripHeaderClick);

function onStripHeaderClick(a, idx){
	//// collapse vertically expanded strip if any
	removeVerticallyExpanded();
	removeVerticallyFadein();
	removeFlyout();
	//// compress if already expanded
	var isAlreadyExpanded = d3.select(this).classed("header_expanded");
	if(isAlreadyExpanded){
		d3.selectAll(".strip_header")
			.classed("header_expanded", false)
			.transition()
			.duration(300)
			.style("width", (w/data.length+3)+"px");
		d3.select(this)
			.select(".strip_header_label")
			.style("-webkit-transform", function(d, i){
				if (i == idx) return "rotate(0deg) translate(0, 0px)";
				else return "rotate(-90deg) translate(0, 0px) ";
			});
		d3.select(d3.selectAll("td.container_table_col")[0][idx])
			.selectAll(".slice_label")
			.transition()
			.style("opacity", 0);
	}else{	
		//// expand strip_header, compress others
		d3.selectAll(".strip_header")
			.transition()
			.duration(300)
			.style("width", function(d, i){ 
				// tag strip header as expanded or not
				d3.select(this).classed("header_expanded", i==idx);
				// tag strip table as expanded or not
				d3.select(d3.selectAll(".strip_table")[0][i]).classed("strip_expanded", i==idx);
				// find all strip labels
				var stripLabel = d3.select(d3.selectAll("td.container_table_col")[0][i])
					.selectAll(".slice_label")
					.transition();
				// if it's the clicked strip, expand and change opacity
				if(i==idx){
					//// refresh chart
					updateChart();
					stripLabel.style("opacity", 1);
					return focusedStrip+"px";
				}else {
					stripLabel.style("opacity", 0);
					return unfocusedStrip+"px"; 
				}
			})
			
			//// rotate expanded strip header label
			.select(".strip_header_label")
			.transition()
			.duration(200)
			.style("-webkit-transform", function(d, i){
				if (i == idx) return "rotate(-1deg) rotate(1deg) translate(10px, -70px)"; // rotation hack
				else return "rotate(-90deg) translate(0, 0px) ";
			});
	}
	
}; 


/******************************** 
* Vertical expansion
*********************************/	
//// expand/collapse vertically on bottom slice click if expanded
d3.selectAll("td.other_slice")
	.on("click", expandVertically);

function expandVertically(d, idx){	
	// check if the corresponding strip header is expanded horizontally
	var isExpandedHoriz = this.parentNode.parentNode.classList.contains("strip_expanded");
	// check if the vertical expansion is already present
	var isExpandedVert = d3.selectAll(".vertical_expansion")[0].length > 0;
	if (isExpandedHoriz && !isExpandedVert){
		// retrieve x position
		var x = this.parentNode.parentNode.parentNode.offsetLeft; 
		//// generate, fade in and expand a new div vertically
		var verticalExpansionHtml = d3.select(this)
			.append("div")
			.attr("class", "vertical_expansion")
			.style("background-color", "silver")
			.style("position", "absolute")
			.style("overflow", "hidden")
			.style("border", "2px dodgerblue solid")
			.style("top", h) // pos absolute w/o .style("left", x)  keep the flow...
			.style("width", focusedStrip+"px")
			.style("height", "0px");
			
		//// collapse down on click and destroy div and table
		verticalExpansionHtml.on("click", removeVerticallyExpanded)
			.transition()
			.style("height", h+"px")
			.style("top", 0);
			
		//// populate the new strip with mock data (redundant)
		var verticalExpansionData = [];
		var verticalExpansionDataSum = 0;
		var verticalExpansionDataMax = 0;
		var verticalExpansionDataMin = 100000;
		var verticalExpansionDataRange = 0;
		for(var i=0; i<10; i++){
			var randomData = Math.round(Math.random()*1000)/100;
			verticalExpansionData.push(randomData);
			verticalExpansionDataSum += randomData;
			if(randomData > verticalExpansionDataMax) verticalExpansionDataMax = randomData;
			if(randomData < verticalExpansionDataMin) verticalExpansionDataMin = randomData;
		}
		verticalExpansionDataRange = verticalExpansionDataMax - verticalExpansionDataMin;
		verticalExpansionData.sort(function(a, b){return a - b;});
		verticalExpansionHtml.append("table")
			.style("border-collapse", "collapse")
			.style("width", "100%")
			.style("height", h+"px")
			.style("table-layout", "fixed")
			.selectAll("tr.flyout_slice")
			.data(verticalExpansionData)
			.enter().append("tr")
			.attr("class", "flyout_slice")
			.append("td")
			.style("overflow", "hidden")
			.style("background-color", function(d, i){
				var grayscale = 250-Math.floor(d/verticalExpansionDataRange*100); 
				return 'rgb('+grayscale+', '+grayscale+', '+grayscale+')'})
			.style("height", function(d, i){
					return (d/verticalExpansionDataSum*100)+"%";})
			.html(function(d, i){return d;});
	}
};

function removeVerticallyExpanded(){
	d3.selectAll(".vertical_expansion")
		.transition()
		.duration(50)
		.style("height", "0px")
		.style("top", h)
		.remove();
};

/******************************** 
* Horizontal flyout
*********************************/	
d3.selectAll(".hierarchy")
	.on("click", flyout);

function flyout(d, idx){
	var isExpandedHoriz = d3.select(this.parentNode.parentNode).classed("strip_expanded");
	if(isExpandedHoriz){
		var that = this;
		//// select container cell
		var parentX = this.parentNode.parentNode.parentNode.offsetLeft;
		var flyoutHtml = d3.select(this.parentNode.parentNode.parentNode)
			.append("div")
			.attr("class", "flyout")
			.style("background-color", "silver")
			.style("position", "absolute")
			.style("border", "2px dodgerblue solid")
			.style("top", "0px")
			.style("width", "0px")
			.style("left", function(){return (parentX+focusedStrip+2)+"px";})
			.style("height", h+"px")
			
		//// delete it on click
		flyoutHtml.on("click", removeFlyout)
			.transition()
			.style("width", (focusedStrip-unfocusedStrip)+"px")
			.style("left", function(){return (parentX+unfocusedStrip+2)+"px";});
			
		//// populate the new strip with mock data
		var flyoutData = [];
		var flyoutDataSum = 0;
		var flyoutDataMax = 0;
		var flyoutDataMin = 100000;
		var flyoutDataRange = 0;
		for(var i=0; i<10; i++){
			var randomData = Math.round(Math.random()*1000)/100;
			flyoutData.push(randomData);
			flyoutDataSum += randomData;
			if(randomData > flyoutDataMax) flyoutDataMax = randomData;
			if(randomData < flyoutDataMin) flyoutDataMin = randomData;
		}
		flyoutDataRange = flyoutDataMax - flyoutDataMin;
		flyoutData.sort(function(a, b){return a - b;});
		flyoutHtml.append("table")
			.style("border-collapse", "collapse")
			.style("width", "100%")
			.style("height", h+"px")
			.style("table-layout", "fixed")
			.selectAll("tr.flyout_slice")
			.data(flyoutData)
			.enter().append("tr")
			.attr("class", "flyout_slice")
			.append("td")
			.style("overflow", "hidden")
			.style("background-color", function(d, i){
				var grayscale = 250-Math.floor(d/flyoutDataRange*100); 
				return 'rgb('+grayscale+', '+grayscale+', '+grayscale+')'})
			.style("height", function(d, i){
					return (d/flyoutDataSum*100)+"%";})
			.html(function(d, i){return d;});
	}
}

//// collapse on header click or a click on another expanded slice
function removeFlyout(){
	d3.selectAll(".flyout")
		.transition()
		.duration(100)
		.style("width", "0px")
		.style("left", function(){return (this.parentNode.offsetLeft+focusedStrip+2)+"px";})
		.remove();
};

/********************************
* Vertical list fade-in
*********************************/
//// expand/collapse vertically on bottom slice click if expanded
d3.selectAll("td.list")
	.on("click", fadeinVertically);

function fadeinVertically(d, idx){
	// check if the corresponding strip header is expanded horizontally
	var isExpandedHoriz = this.parentNode.parentNode.classList.contains("strip_expanded");
	// check if the vertical expansion is already present
	var isExpandedVert = d3.selectAll(".vertical_fadein")[0].length > 0;
	if (isExpandedHoriz && !isExpandedVert){
		// retrieve x position
		var x = this.parentNode.parentNode.parentNode.offsetLeft;
		//// generate, fade in and expand a new div vertically
		var verticalFadeinHtml = d3.select(this)
			.append("div")
			.attr("class", "vertical_fadein")
			.style("background-color", "white")
			.style("position", "absolute")
			.style("overflow", "hidden")
			.style("border", "2px dodgerblue solid")
			.style("top", 0)
			.style("width", focusedStrip+"px")
			.style("height", h+"px")
			.style("opacity", 0);

	    //// collapse down on click and destroy div and table
		verticalFadeinHtml.on("click", removeVerticallyFadein)
			.transition()
			.style("opacity", 1);

		//// populate with mock data
		var datafadein = ["America 80%", "Europe 12%", "Asia 8%"];
		verticalFadeinHtml.append("table")
			.style("border", "none")
			.style("width", "100%")
			.style("height", h+"px")
			.style("table-layout", "fixed")
			.html("<tr><td>&nbsp</td></tr><tr><td></td>America 80%<br />Europe 12%<br />Asia 8%</tr>");
	}
};

function removeVerticallyFadein(){
	d3.selectAll(".vertical_fadein")
		.transition()
		.duration(50)
		.style("opacity", 0)
		.remove();
};

/********************************
* Main pane
*********************************/
var mainPane = d3.select("#viz")
	.append("div")
	.attr("id", "pane")
	.style("width", "340px")
	.style("height", "518px")
	.style("float", "left");

//// bar chart
var chartH = 400;
var chartW = 270;
var chart = mainPane.append("div")
	.attr("id", "chart")
	.style("border", "2px solid black")
	.style("background-color", "white")
	.style("margin", "40px")
	.style("height", chartH+"px")
	.style("width", chartW+"px");

var chartLabel = mainPane.append("div")
    .style("margin-top", "-40px")
    .style("margin-left", "40px")
	.style("height", "30px")
	.style("width", "500px")
    .style("font-size", "18px")
    .html('<br><span style="color: gray; font-size: 18px">Cars Dataset</span>');

function updateChart(){
    var dataFromStrip = d3.select(".strip_expanded")[0][0].__data__;
    var stripLabel = d3.select(".header_expanded").text();
    var chartData = [];
    for(var i=0; i<dataFromStrip.length; i++){
        chartData.push(dataFromStrip[i].percent);
    }
	var barMargin = 10;
	var barW = Math.floor((chartW+barMargin) / (chartData.length))-barMargin-5;
	var chartDataMax = d3.max(chartData);

	var chartToUpdate = chart.selectAll("div.bar")
		.data(chartData);

	chartToUpdate.exit().transition()
	    .style("opacity", 0)
	    .remove();
	    		
	var test = chartToUpdate.enter().append("div")
		.attr("class", "bar")
		.style("position", "relative")
		.style("float", "left")
		.style("margin-left", barMargin+"px")
		.style("background-color", "deepskyblue")
		.style("width", "0px")
		.append("div")
        .attr("id", "bar_label");

    chartToUpdate.transition()
	    .delay(500)
	    .style("width", barW+"px")
		.style("height", function(d, i){return (d/chartDataMax*chartH-20)+10+"px"})
		.style("top", function(d, i){return chartH+20-(d/chartDataMax*chartH)-10+"px"})
		.select("div")
		.text(function(d, i){return d+"%"});

    chartLabel.html(stripLabel+'<br><span style="color: gray; font-size: 18px">Cars Dataset</span>');
}